@@ -1,6 +1,6 @@
 /*
  * Mar  8, 2000 by Hajimu UMEMOTO <ume@mahoroba.org>
- * $Id: gai.h,v 1.6 2003/07/25 16:46:20 rjs3 Exp $
+ * $Id: gai.h,v 1.8 2006/04/10 13:36:20 mel Exp $
  *
  * This module is besed on ssh-1.2.27-IPv6-1.5 written by
  * KIKUCHI Takahiro <kick@kyoto.wide.ad.jp>
@@ -85,13 +85,6 @@ struct addrinfo {
 };
 #endif
 
-#ifndef AI_NUMERICHOST   /* support glibc 2.0.x */
-#define	AI_NUMERICHOST	4
-#define NI_NUMERICHOST	2
-#define NI_NAMEREQD	4
-#define NI_NUMERICSERV	8
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -101,7 +94,7 @@ int	getnameinfo(const struct sockaddr *, socklen_t, char *,
 		    size_t, char *, size_t, int);
 #endif
 
-#ifndef HAVE_GETNAMEINFO
+#ifndef HAVE_GETADDRINFO
 int	getaddrinfo(const char *, const char *,
 		    const struct addrinfo *, struct addrinfo **);
 void	freeaddrinfo(struct addrinfo *);
