@@ -1,9 +1,10 @@
 /* common.c - Functions that are common to server and clinet
  * Rob Siemborski
  * Tim Martin
+ * $Id: common.c,v 1.114 2006/04/19 18:39:59 mel Exp $
  */
 /* 
- * Copyright (c) 1998-2016 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -21,13 +22,12 @@
  *    endorse or promote products derived from this software without
  *    prior written permission. For permission or any other legal
  *    details, please contact  
+ *      Office of Technology Transfer
  *      Carnegie Mellon University
- *      Center for Technology Transfer and Enterprise Creation
- *      4615 Forbes Avenue
- *      Suite 302
- *      Pittsburgh, PA  15213
- *      (412) 268-7393, fax: (412) 268-7395
- *      innovation@andrew.cmu.edu
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
  *
  * 4. Redistributions of any form whatsoever must retain the following
  *    acknowledgment:
@@ -82,12 +82,14 @@ static char * _sasl_get_default_unix_path(void *context __attribute__((unused)),
 #else
 /* NB: Always returned allocated value */
 static char * _sasl_get_default_win_path(void *context __attribute__((unused)),
-                            TCHAR * reg_attr_name, char * default_value);
+                            char * reg_attr_name, char * default_value);
 #endif
 
 
+static const char build_ident[] = "$Build: libsasl " PACKAGE "-" VERSION " $";
+
 /* It turns out to be convenient to have a shared sasl_utils_t */
-const sasl_utils_t *sasl_global_utils = NULL;
+LIBSASL_VAR const sasl_utils_t *sasl_global_utils = NULL;
 
 /* Should be a null-terminated array that lists the available mechanisms */
 static char **global_mech_list = NULL;
@@ -105,27 +107,20 @@ sasl_allocation_utils_t _sasl_allocation_utils={
   (sasl_realloc_t *) &realloc,
   (sasl_free_t *) &free
 };
-int _sasl_allocation_locked = 0;
 
 #define SASL_ENCODEV_EXTRA  4096
 
 /* Default getpath/getconfpath callbacks. These can be edited by sasl_set_path(). */
 static sasl_callback_t default_getpath_cb = {
-    SASL_CB_GETPATH, (sasl_callback_ft)&_sasl_getpath, NULL
+    SASL_CB_GETPATH, &_sasl_getpath, NULL
 };
 static sasl_callback_t default_getconfpath_cb = {
-    SASL_CB_GETCONFPATH, (sasl_callback_ft)&_sasl_getconfpath, NULL
+    SASL_CB_GETCONFPATH, &_sasl_getconfpath, NULL
 };
 
 static char * default_plugin_path = NULL;
 static char * default_conf_path = NULL;
 
-static int _sasl_global_getopt(void *context,
-			       const char *plugin_name,
-			       const char *option,
-			       const char ** result,
-			       unsigned *len);
- 
 /* Intenal mutex functions do as little as possible (no thread protection) */
 static void *sasl_mutex_alloc(void)
 {
@@ -154,21 +149,13 @@ sasl_mutex_utils_t _sasl_mutex_utils={
   &sasl_mutex_free
 };
 
-void sasl_set_mutex(sasl_mutex_alloc_t *n,
-		    sasl_mutex_lock_t *l,
-		    sasl_mutex_unlock_t *u,
-		    sasl_mutex_free_t *d)
+void sasl_set_mutex(sasl_mutex_alloc_t *n, sasl_mutex_lock_t *l,
+		    sasl_mutex_unlock_t *u, sasl_mutex_free_t *d)
 {
-    /* Disallow mutex function changes once sasl_client_init
-       and/or sasl_server_init is called */
-    if (_sasl_server_cleanup_hook || _sasl_client_cleanup_hook) {
-	return;
-    }
-
-    _sasl_mutex_utils.alloc=n;
-    _sasl_mutex_utils.lock=l;
-    _sasl_mutex_utils.unlock=u;
-    _sasl_mutex_utils.free=d;
+  _sasl_mutex_utils.alloc=n;
+  _sasl_mutex_utils.lock=l;
+  _sasl_mutex_utils.unlock=u;
+  _sasl_mutex_utils.free=d;
 }
 
 /* copy a string to malloced memory */
@@ -191,10 +178,10 @@ int _sasl_add_string(char **out, size_t *alloclen,
   if (add==NULL) add = "(null)";
 
   addlen=strlen(add); /* only compute once */
-  if (_buf_alloc(out, alloclen, (*outlen)+addlen+1)!=SASL_OK)
+  if (_buf_alloc(out, alloclen, (*outlen)+addlen)!=SASL_OK)
     return SASL_NOMEM;
 
-  strcpy(*out + *outlen, add);
+  strncpy(*out + *outlen, add, addlen);
   *outlen += addlen;
 
   return SASL_OK;
@@ -227,7 +214,7 @@ int sasl_set_path (int path_type, char * path)
             }
 
             /* Update the default getpath_t callback */
-            default_getpath_cb.proc = (sasl_callback_ft)&_sasl_getpath_simple;
+            default_getpath_cb.proc = (int (*)()) &_sasl_getpath_simple;
             break;
 
         case SASL_PATH_TYPE_CONFIG:
@@ -241,7 +228,7 @@ int sasl_set_path (int path_type, char * path)
             }
 
             /* Update the default getpath_t callback */
-            default_getconfpath_cb.proc = (sasl_callback_ft)&_sasl_getconfpath_simple;
+            default_getconfpath_cb.proc = (int (*)()) &_sasl_getconfpath_simple;
             break;
 
         default:
@@ -253,10 +240,7 @@ int sasl_set_path (int path_type, char * path)
 
 /* return the version of the cyrus sasl library as compiled,
  * using 32 bits: high byte is major version, second byte is minor version,
- * low 16 bits are step #.
- * Patch version is not available using this function,
- * use sasl_version_info() instead.
- */
+ * low 16 bits are step # */
 void sasl_version(const char **implementation, int *version) 
 {
     if(implementation) *implementation = implementation_string;
@@ -384,7 +368,7 @@ _sasl_encodev (sasl_conn_t *conn,
         conn->multipacket_encoded_data.curlen += *outputlen;
 
         *output = conn->multipacket_encoded_data.data;
-        *outputlen = (unsigned)conn->multipacket_encoded_data.curlen;
+        *outputlen = conn->multipacket_encoded_data.curlen;
     }
 
     (*p_num_packets)++;
@@ -400,7 +384,7 @@ int sasl_encodev(sasl_conn_t *conn,
 		 const char **output,
                  unsigned *outputlen)
 {
-    int result = SASL_OK;
+    int result;
     unsigned i;
     unsigned j;
     size_t total_size = 0;
@@ -408,7 +392,7 @@ int sasl_encodev(sasl_conn_t *conn,
     struct iovec last_invec;
     unsigned cur_numiov;
     char * next_buf = NULL;
-    size_t remainder_len;
+    unsigned remainder_len;
     unsigned index_offset;
     unsigned allocated = 0;
     /* Number of generated SASL packets */
@@ -480,7 +464,7 @@ int sasl_encodev(sasl_conn_t *conn,
 
             if (next_buf != NULL) {
                 cur_invec[0].iov_base = next_buf;
-                cur_invec[0].iov_len = (long)remainder_len;
+                cur_invec[0].iov_len = remainder_len;
                 cur_numiov++;
                 index_offset = 1;
             } else {
@@ -513,9 +497,7 @@ int sasl_encodev(sasl_conn_t *conn,
                the conn->oparams.maxoutbuf buffer. */
             /* Note, if next_buf points to the very end of the IOV record,
                it will be reset to NULL below */
-            /* Note, that some platforms define iov_base as "void *",
-               thus the typecase below */
-            next_buf = (char *) last_invec.iov_base + last_invec.iov_len;
+            next_buf = last_invec.iov_base + last_invec.iov_len;
             /* Note - remainder_len is how many bytes left to be encoded in
                the current IOV slot. */
             remainder_len = (total_size + invec[i].iov_len) - conn->oparams.maxoutbuf;
@@ -531,9 +513,7 @@ int sasl_encodev(sasl_conn_t *conn,
 
                 /* Note, if next_buf points to the very end of the IOV record,
                    it will be reset to NULL below */
-                /* Note, that some platforms define iov_base as "void *",
-                   thus the typecase below */
-                next_buf = (char *) last_invec.iov_base + last_invec.iov_len;
+                next_buf = last_invec.iov_base + last_invec.iov_len;
                 remainder_len = remainder_len - conn->oparams.maxoutbuf;
 
                 result = _sasl_encodev (conn,
@@ -564,7 +544,7 @@ int sasl_encodev(sasl_conn_t *conn,
     /* Force encoding of any partial buffer. Might not be optimal on the wire. */
     if (next_buf != NULL) {
         last_invec.iov_base = next_buf;
-        last_invec.iov_len = (long)remainder_len;
+        last_invec.iov_len = remainder_len;
 
         result = _sasl_encodev (conn,
 	                        &last_invec,
@@ -657,16 +637,28 @@ sasl_set_alloc(sasl_malloc_t *m,
 	       sasl_realloc_t *r,
 	       sasl_free_t *f)
 {
-  if (_sasl_allocation_locked++)  return;
-
   _sasl_allocation_utils.malloc=m;
   _sasl_allocation_utils.calloc=c;
   _sasl_allocation_utils.realloc=r;
   _sasl_allocation_utils.free=f;
 }
 
-void sasl_common_done(void)
+void sasl_done(void)
 {
+    if (_sasl_server_cleanup_hook && _sasl_server_cleanup_hook() == SASL_OK) {
+	_sasl_server_idle_hook = NULL;
+	_sasl_server_cleanup_hook = NULL;
+    }
+    
+    if (_sasl_client_cleanup_hook && _sasl_client_cleanup_hook() == SASL_OK) {
+	_sasl_client_idle_hook = NULL;	
+	_sasl_client_cleanup_hook = NULL;
+    }
+    
+    if (_sasl_server_cleanup_hook || _sasl_client_cleanup_hook) {
+	return;
+    }
+
     /* NOTE - the caller will need to reinitialize the values,
        if it is going to call sasl_client_init/sasl_server_init again. */
     if (default_plugin_path != NULL) {
@@ -686,30 +678,8 @@ void sasl_common_done(void)
     
     _sasl_free_utils(&sasl_global_utils);
     
-    if (global_mech_list) {
-	sasl_FREE(global_mech_list);
-	global_mech_list = NULL;
-    }
-}
-
-/* This function is for backward compatibility */
-void sasl_done(void)
-{
-    if (_sasl_server_cleanup_hook && _sasl_server_cleanup_hook() == SASL_OK) {
-	_sasl_server_idle_hook = NULL;
-	_sasl_server_cleanup_hook = NULL;
-    }
-    
-    if (_sasl_client_cleanup_hook && _sasl_client_cleanup_hook() == SASL_OK) {
-	_sasl_client_idle_hook = NULL;	
-	_sasl_client_cleanup_hook = NULL;
-    }
-    
-    if (_sasl_server_cleanup_hook || _sasl_client_cleanup_hook) {
-	return;
-    }
-
-    sasl_common_done();
+    if(global_mech_list) sasl_FREE(global_mech_list);
+    global_mech_list = NULL;
 }
 
 /* fills in the base sasl_conn_t info */
@@ -770,14 +740,11 @@ int _sasl_conn_init(sasl_conn_t *conn,
 
   if(serverFQDN) {
       result = _sasl_strdup(serverFQDN, &conn->serverFQDN, NULL);
-      sasl_strlower (conn->serverFQDN);
   } else if (conn->type == SASL_CONN_SERVER) {
       /* We can fake it because we *are* the server */
-      char name[MAXFQDNLEN];
+      char name[MAXHOSTNAMELEN];
       memset(name, 0, sizeof(name));
-      if (get_fqhostname (name, MAXFQDNLEN, 0) != 0) {
-        return (SASL_FAIL);
-      }
+      gethostname(name, MAXHOSTNAMELEN);
       
       result = _sasl_strdup(name, &conn->serverFQDN, NULL);
   } else {
@@ -793,19 +760,7 @@ int _sasl_conn_init(sasl_conn_t *conn,
 int _sasl_common_init(sasl_global_callbacks_t *global_callbacks)
 {
     int result;
-
-    /* The last specified global callback always wins */
-    if (sasl_global_utils != NULL) {
-	sasl_utils_t * global_utils = (sasl_utils_t *)sasl_global_utils;
-	global_utils->getopt = &_sasl_global_getopt;
-	global_utils->getopt_context = global_callbacks;
-    }
-
-    /* Do nothing if we are already initialized */
-    if (free_mutex) {
-	return SASL_OK;
-    }
-
+    
     /* Setup the global utilities */
     if(!sasl_global_utils) {
 	sasl_global_utils = _sasl_alloc_utils(NULL, global_callbacks);
@@ -816,9 +771,8 @@ int _sasl_common_init(sasl_global_callbacks_t *global_callbacks)
     result = sasl_canonuser_add_plugin("INTERNAL", internal_canonuser_init);
     if(result != SASL_OK) return result;    
 
-    if (!free_mutex) {
+    if (!free_mutex)
 	free_mutex = sasl_MUTEX_ALLOC();
-    }
     if (!free_mutex) return SASL_FAIL;
 
     return SASL_OK;
@@ -836,20 +790,15 @@ void sasl_dispose(sasl_conn_t **pconn)
 
   /* serialize disposes. this is necessary because we can't
      dispose of conn->mutex if someone else is locked on it */
-  if (!free_mutex) {
-      free_mutex = sasl_MUTEX_ALLOC();
-      if (!free_mutex) return;
-  }
-
   result = sasl_MUTEX_LOCK(free_mutex);
   if (result!=SASL_OK) return;
   
   /* *pconn might have become NULL by now */
-  if (*pconn) {
-      (*pconn)->destroy_conn(*pconn);
-      sasl_FREE(*pconn);
-      *pconn=NULL;
-  }
+  if (! (*pconn)) return;
+
+  (*pconn)->destroy_conn(*pconn);
+  sasl_FREE(*pconn);
+  *pconn=NULL;
 
   sasl_MUTEX_UNLOCK(free_mutex);
 }
@@ -896,7 +845,7 @@ void _sasl_conn_dispose(sasl_conn_t *conn) {
  * returns:
  *  SASL_OK       -- no error
  *  SASL_NOTDONE  -- property not available yet
- *  SASL_BADPARAM -- bad property number or SASL context is NULL
+ *  SASL_BADPARAM -- bad property number
  */
 int sasl_getprop(sasl_conn_t *conn, int propnum, const void **pvalue)
 {
@@ -916,7 +865,7 @@ int sasl_getprop(sasl_conn_t *conn, int propnum, const void **pvalue)
       *(unsigned **)pvalue = &conn->oparams.maxoutbuf;
       break;
   case SASL_GETOPTCTX:
-      result = _sasl_getcallback(conn, SASL_CB_GETOPT, (sasl_callback_ft *)&getopt, &context);
+      result = _sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context);
       if(result != SASL_OK) break;
       
       *(void **)pvalue = context;
@@ -1021,18 +970,6 @@ int sasl_getprop(sasl_conn_t *conn, int propnum, const void **pvalue)
       else
 	  *((const char **)pvalue) = conn->oparams.client_creds;
       break;
-  case SASL_GSS_PEER_NAME:
-      if(! conn->oparams.gss_peer_name)
-	  result = SASL_NOTDONE;
-      else
-	  *((const char **)pvalue) = conn->oparams.gss_peer_name;
-      break;
-  case SASL_GSS_LOCAL_NAME:
-      if(! conn->oparams.gss_local_name)
-	  result = SASL_NOTDONE;
-      else
-	  *((const char **)pvalue) = conn->oparams.gss_local_name;
-      break;
   case SASL_SSF_EXTERNAL:
       *((const sasl_ssf_t **)pvalue) = &conn->external.ssf;
       break;
@@ -1042,23 +979,6 @@ int sasl_getprop(sasl_conn_t *conn, int propnum, const void **pvalue)
   case SASL_SEC_PROPS:
       *((const sasl_security_properties_t **)pvalue) = &conn->props;
       break;
-  case SASL_GSS_CREDS:
-      if(conn->type == SASL_CONN_CLIENT)
-	  *(const void **)pvalue = 
-              ((sasl_client_conn_t *)conn)->cparams->gss_creds;
-      else
-	  *(const void **)pvalue = 
-              ((sasl_server_conn_t *)conn)->sparams->gss_creds;
-      break;
-  case SASL_HTTP_REQUEST: {
-      if (conn->type == SASL_CONN_SERVER)
-	  *(const sasl_http_request_t **)pvalue =
-	      ((sasl_server_conn_t *)conn)->sparams->http_request;
-      else
-	  *(const sasl_http_request_t **)pvalue =
-	      ((sasl_client_conn_t *)conn)->cparams->http_request;
-      break;
-  }
   default: 
       result = SASL_BADPARAM;
   }
@@ -1270,33 +1190,6 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
       }
       break;
 
-  case SASL_GSS_CREDS:
-      if(conn->type == SASL_CONN_CLIENT)
-          ((sasl_client_conn_t *)conn)->cparams->gss_creds = value;
-      else
-          ((sasl_server_conn_t *)conn)->sparams->gss_creds = value;
-      break;
-
-  case SASL_CHANNEL_BINDING: {
-    const struct sasl_channel_binding *cb = (const struct sasl_channel_binding *)value;
-
-    if (conn->type == SASL_CONN_SERVER)
-        ((sasl_server_conn_t *)conn)->sparams->cbinding = cb;
-    else
-        ((sasl_client_conn_t *)conn)->cparams->cbinding = cb;
-    break;
-  }
-
-  case SASL_HTTP_REQUEST: {
-      const sasl_http_request_t *req = (const sasl_http_request_t *)value;
-
-      if (conn->type == SASL_CONN_SERVER)
-	  ((sasl_server_conn_t *)conn)->sparams->http_request = req;
-      else
-	  ((sasl_client_conn_t *)conn)->cparams->http_request = req;
-      break;
-  }
-
   default:
       sasl_seterror(conn, 0, "Unknown parameter type");
       result = SASL_BADPARAM;
@@ -1331,11 +1224,11 @@ const char *sasl_errstring(int saslerr,
     case SASL_BUFOVER:  return "overflowed buffer";
     case SASL_NOMECH:   return "no mechanism available";
     case SASL_BADPROT:  return "bad protocol / cancel";
-    case SASL_NOTDONE:  return "can't request information until later in exchange";
+    case SASL_NOTDONE:  return "can't request info until later in exchange";
     case SASL_BADPARAM: return "invalid parameter supplied";
     case SASL_TRYAGAIN: return "transient failure (e.g., weak key)";
     case SASL_BADMAC:   return "integrity check failed";
-    case SASL_NOTINIT:  return "SASL library is not initialized";
+    case SASL_NOTINIT:  return "SASL library not initialized";
                              /* -- client only codes -- */
     case SASL_INTERACT:   return "needs user interaction";
     case SASL_BADSERV:    return "server failed mutual authentication step";
@@ -1356,12 +1249,6 @@ const char *sasl_errstring(int saslerr,
     case SASL_NOCHANGE:   return "requested change was not needed";
     case SASL_WEAKPASS:   return "passphrase is too weak for security policy";
     case SASL_NOUSERPASS: return "user supplied passwords are not permitted";
-    case SASL_NEED_OLD_PASSWD: return "sasl_setpass needs old password in order "
-				"to perform password change";
-    case SASL_CONSTRAINT_VIOLAT: return "sasl_setpass can't store a property because "
-			        "of a constraint violation";
-    case SASL_BADBINDING: return "channel binding failure";
-    case SASL_CONFIGERR:  return "error when parsing configuration file";
 
     default:   return "undefined error!";
     }
@@ -1383,9 +1270,7 @@ const char *sasl_errdetail(sasl_conn_t *conn)
 	     sasl_usererr(conn->error_code), errstr);
     
     need_len = (unsigned) (strlen(leader) + strlen(conn->error_buf) + 12);
-    if (_buf_alloc(&conn->errdetail_buf, &conn->errdetail_buf_len, need_len) != SASL_OK) {
-        return NULL;
-    }
+    _buf_alloc(&conn->errdetail_buf, &conn->errdetail_buf_len, need_len);
 
     snprintf(conn->errdetail_buf, need_len, "%s%s", leader, conn->error_buf);
    
@@ -1527,9 +1412,12 @@ _sasl_getsimple(void *context,
 		size_t *len)
 {
   const char *userid;
+  sasl_conn_t *conn;
 
   if (! context || ! result) return SASL_BADPARAM;
 
+  conn = (sasl_conn_t *)context;
+
   switch(id) {
   case SASL_CB_AUTHNAME:
     userid = getenv("USER");
@@ -1550,12 +1438,10 @@ _sasl_getsimple(void *context,
 	DWORD i;
 	BOOL rval;
 	static char sender[128];
-
-    TCHAR tsender[128];
-	i = sizeof(tsender) / sizeof(tsender[0]);
-	rval = GetUserName(tsender, &i);
+	
+	i = sizeof(sender);
+	rval = GetUserName(sender, &i);
 	if ( rval) { /* got a userid */
-        WideCharToMultiByte(CP_UTF8, 0, tsender, -1, sender, sizeof(sender), NULL, NULL); /* -1 ensures null-terminated utf8 */
 		*result = sender;
 		if (len) *len = strlen(sender);
 		return SASL_OK;
@@ -1678,7 +1564,6 @@ _sasl_getconfpath_simple(void *context __attribute__((unused)),
     return SASL_OK;
 }
 
-
 static int
 _sasl_verifyfile(void *context __attribute__((unused)),
 		 char *file  __attribute__((unused)),
@@ -1716,7 +1601,7 @@ _sasl_proxy_policy(sasl_conn_t *conn,
 
 int _sasl_getcallback(sasl_conn_t * conn,
 		      unsigned long callbackid,
-		      sasl_callback_ft *pproc,
+		      int (**pproc)(),
 		      void **pcontext)
 {
   const sasl_callback_t *callback;
@@ -1731,10 +1616,10 @@ int _sasl_getcallback(sasl_conn_t * conn,
       INTERROR(conn, SASL_FAIL);
   case SASL_CB_GETOPT:
       if (conn) {
-	  *pproc = (sasl_callback_ft)&_sasl_conn_getopt;
+	  *pproc = &_sasl_conn_getopt;
 	  *pcontext = conn;
       } else {
-	  *pproc = (sasl_callback_ft)&_sasl_global_getopt;
+	  *pproc = &_sasl_global_getopt;
 	  *pcontext = NULL;
       }
       return SASL_OK;
@@ -1779,7 +1664,7 @@ int _sasl_getcallback(sasl_conn_t * conn,
   switch (callbackid) {
 #ifdef HAVE_SYSLOG
   case SASL_CB_LOG:
-    *pproc = (sasl_callback_ft)&_sasl_syslog;
+    *pproc = (int (*)()) &_sasl_syslog;
     *pcontext = conn;
     return SASL_OK;
 #endif /* HAVE_SYSLOG */
@@ -1792,15 +1677,15 @@ int _sasl_getcallback(sasl_conn_t * conn,
     *pcontext = default_getconfpath_cb.context;
     return SASL_OK;
   case SASL_CB_AUTHNAME:
-    *pproc = (sasl_callback_ft)&_sasl_getsimple;
+    *pproc = (int (*)()) &_sasl_getsimple;
     *pcontext = conn;
     return SASL_OK;
   case SASL_CB_VERIFYFILE:
-    *pproc = (sasl_callback_ft)&_sasl_verifyfile;
+    *pproc = & _sasl_verifyfile;
     *pcontext = NULL;
     return SASL_OK;
   case SASL_CB_PROXY_POLICY:
-    *pproc = (sasl_callback_ft)&_sasl_proxy_policy;
+    *pproc = (int (*)()) &_sasl_proxy_policy;
     *pcontext = NULL;
     return SASL_OK;
   }
@@ -1828,7 +1713,7 @@ _sasl_log (sasl_conn_t *conn,
 	   const char *fmt,
 	   ...)
 {
-  char *out = NULL;
+  char *out=(char *) sasl_ALLOC(250);
   size_t alloclen=100; /* current allocated length */
   size_t outlen=0; /* current length of output buffer */
   size_t formatlen;
@@ -1842,15 +1727,13 @@ _sasl_log (sasl_conn_t *conn,
   char *cval;
   va_list ap; /* varargs thing */
 
-  if(!fmt) return;
-  
-  out = (char *) sasl_ALLOC(250);
+  if(!fmt) goto done;
   if(!out) return;
-
+  
   formatlen = strlen(fmt);
 
   /* See if we have a logging callback... */
-  result = _sasl_getcallback(conn, SASL_CB_LOG, (sasl_callback_ft *)&log_cb, &log_ctx);
+  result = _sasl_getcallback(conn, SASL_CB_LOG, &log_cb, &log_ctx);
   if (result == SASL_OK && ! log_cb)
     result = SASL_FAIL;
   if (result != SASL_OK) goto done;
@@ -1983,11 +1866,12 @@ _sasl_log (sasl_conn_t *conn,
   if (result != SASL_OK) goto done;
   out[outlen]=0;
 
+  va_end(ap);    
+
   /* send log message */
   result = log_cb(log_ctx, level, out);
 
  done:
-  va_end(ap);
   if(out) sasl_FREE(out);
 }
 
@@ -2157,7 +2041,7 @@ _sasl_find_verifyfile_callback(const sasl_callback_t *callbacks)
 {
   static const sasl_callback_t default_verifyfile_cb = {
     SASL_CB_VERIFYFILE,
-    (sasl_callback_ft)&_sasl_verifyfile,
+    &_sasl_verifyfile,
     NULL
   };
 
@@ -2359,7 +2243,6 @@ int _sasl_build_mechlist(void)
     }
 
     if(!olist) {
-	/* This is not going to be very useful */
 	printf ("no olist");
 	return SASL_FAIL;
     }
@@ -2415,29 +2298,6 @@ int sasl_listmech(sasl_conn_t *conn,
     PARAMERROR(conn);
 }
 
-int _sasl_is_equal_mech(const char *req_mech,
-                        const char *plug_mech,
-			size_t req_mech_len,
-                        int *plus)
-{
-    size_t n;
-
-    if (req_mech_len > 5 &&
-        strcasecmp(&req_mech[req_mech_len - 5], "-PLUS") == 0) {
-        n = req_mech_len - 5;
-        *plus = 1;
-    } else {
-        n = req_mech_len;
-        *plus = 0;
-    }
-
-    if (n < strlen(plug_mech)) {
-	/* Don't allow arbitrary prefix match */
-	return 0;
-    }
-
-    return (strncasecmp(req_mech, plug_mech, n) == 0);
-}
 
 #ifndef WIN32
 static char *
@@ -2458,11 +2318,11 @@ _sasl_get_default_unix_path(void *context __attribute__((unused)),
     return path;
 }
 
-#else /*WIN32*/
+#else
 /* Return NULL on failure */
 static char *
 _sasl_get_default_win_path(void *context __attribute__((unused)),
-                           TCHAR * reg_attr_name,
+                           char * reg_attr_name,
                            char * default_value)
 {
     /* Open registry entry, and find all registered SASL libraries.
@@ -2482,12 +2342,12 @@ _sasl_get_default_win_path(void *context __attribute__((unused)),
     HKEY  hKey;
     DWORD ret;
     DWORD ValueType;		    /* value type */
-    DWORD cbData;		    /* value size in bytes and later number of wchars */
-    TCHAR * ValueData;		    /* value */
-    DWORD cbExpandedData;	    /* "expanded" value size in wchars */
-    TCHAR * ExpandedValueData;	    /* "expanded" value */
-    TCHAR * return_value;	    /* function return value */
-    TCHAR * tmp;
+    DWORD cbData;		    /* value size */
+    BYTE * ValueData;		    /* value */
+    DWORD cbExpandedData;	    /* "expanded" value size */
+    BYTE * ExpandedValueData;	    /* "expanded" value */
+    char * return_value;	    /* function return value */
+    char * tmp;
 
     /* Initialization */
     ExpandedValueData = NULL;
@@ -2503,9 +2363,8 @@ _sasl_get_default_win_path(void *context __attribute__((unused)),
 
     if (ret != ERROR_SUCCESS) { 
         /* no registry entry */
-        char *ret;
-        (void) _sasl_strdup (default_value, &ret, NULL);
-        return ret;
+        (void) _sasl_strdup (default_value, &return_value, NULL);
+        return return_value;
     }
 
     /* figure out value type and required buffer size */
@@ -2526,36 +2385,29 @@ _sasl_get_default_win_path(void *context __attribute__((unused)),
     }
 
     /* Any high water mark? */
-    ValueData = sasl_ALLOC(cbData + 2 * sizeof(TCHAR)); /* extra bytes to insert null-terminator if it's missed */
+    ValueData = sasl_ALLOC(cbData);
     if (ValueData == NULL) {
 	return_value = NULL;
 	goto CLEANUP;
     };
 
-    if (RegQueryValueEx(hKey,
-        reg_attr_name,
-        NULL,	    /* reserved */
-        &ValueType,
-        (LPBYTE)ValueData,
-        &cbData) != ERROR_SUCCESS) {
-        return_value = NULL;
-        goto CLEANUP;
-    }
-    cbData /= sizeof(TCHAR); /* covert to number of symbols */
-    ValueData[cbData] = '\0'; /* MS docs say we have to to that */
-    ValueData[cbData + 1] = '\0'; /* for MULTI */
+    RegQueryValueEx (hKey,
+		     reg_attr_name,
+		     NULL,	    /* reserved */
+		     &ValueType,
+		     ValueData,
+		     &cbData);
 
     switch (ValueType) {
     case REG_EXPAND_SZ:
         /* : A random starting guess */
         cbExpandedData = cbData + 1024;
-        ExpandedValueData = (TCHAR*)sasl_ALLOC(cbExpandedData * sizeof(TCHAR));
+        ExpandedValueData = sasl_ALLOC(cbExpandedData);
         if (ExpandedValueData == NULL) {
             return_value = NULL;
             goto CLEANUP;
         };
 
-
         cbExpandedData = ExpandEnvironmentStrings(
                                                   ValueData,
                                                   ExpandedValueData,
@@ -2570,7 +2422,7 @@ _sasl_get_default_win_path(void *context __attribute__((unused)),
         /* : Must retry expansion with the bigger buffer */
         if (cbExpandedData > cbData + 1024) {
             /* : Memory leak here if can't realloc */
-            ExpandedValueData = sasl_REALLOC(ExpandedValueData, cbExpandedData * sizeof(TCHAR));
+            ExpandedValueData = sasl_REALLOC(ExpandedValueData, cbExpandedData);
             if (ExpandedValueData == NULL) {
                 return_value = NULL;
                 goto CLEANUP;
@@ -2611,7 +2463,7 @@ _sasl_get_default_win_path(void *context __attribute__((unused)),
                 /* : Replace delimiting NUL with our delimiter characted */
                 tmp[0] = PATHS_DELIMITER;
             }
-            tmp += (_tcslen(tmp));
+            tmp += strlen(tmp);
         }
         break;
 
@@ -2624,51 +2476,15 @@ _sasl_get_default_win_path(void *context __attribute__((unused)),
         goto CLEANUP;
     }
 
-    return_value = ValueData; /* just to flag we have a result */
+    return_value = ValueData;
 
 CLEANUP:
     RegCloseKey(hKey);
     if (ExpandedValueData != NULL) sasl_FREE(ExpandedValueData);
     if (return_value == NULL) {
-	    if (ValueData != NULL) sasl_FREE(ValueData);
-        return NULL;
-    }
-    if (sizeof(TCHAR) == sizeof(char)) {
-        return (char*)return_value;
+	if (ValueData != NULL) sasl_FREE(ValueData);
     }
 
-    /* convert to utf-8 for compatibility with other OS' */
-    {
-        char *tmp = _sasl_wchar_to_utf8(return_value);
-        sasl_FREE(return_value);
-        return tmp;
-    }
+    return (return_value);
 }
-
-char* _sasl_wchar_to_utf8(WCHAR *str)
-{
-    size_t bufLen = WideCharToMultiByte(CP_UTF8, 0, str, -1, NULL, 0, NULL, NULL);
-    char *buf = sasl_ALLOC(bufLen);
-    if (buf) {
-        if (WideCharToMultiByte(CP_UTF8, 0, str, -1, buf, bufLen, NULL, NULL) == 0) { /* -1 ensures null-terminated utf8 */
-            sasl_FREE(buf);
-            buf = NULL;
-        }
-    }
-    return buf;
-}
-
-WCHAR* _sasl_utf8_to_wchar(const char *str)
-{
-    size_t bufLen = MultiByteToWideChar(CP_UTF8, 0, str, -1, NULL, 0);
-    WCHAR *buf = sasl_ALLOC(bufLen * sizeof(WCHAR));
-    if (buf) {
-        if (MultiByteToWideChar(CP_UTF8, 0, str, -1, buf, bufLen) == 0) { /* -1 ensures null-terminated utf8 */
-            sasl_FREE(buf);
-            buf = NULL;
-        }
-    }
-    return buf;
-}
-
-#endif /*WIN32*/
+#endif
