@@ -1,10 +1,10 @@
 /* saslutil.c
  * Rob Siemborski
  * Tim Martin
- * $Id: saslutil.c,v 1.39 2002/11/26 20:01:22 leg Exp $
+ * $Id: saslutil.c,v 1.44 2006/03/13 18:26:36 mel Exp $
  */
 /* 
- * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -47,7 +47,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -169,48 +168,71 @@ int sasl_encode64(const char *_in, unsigned inlen,
  * returns:
  * SASL_BADPROT on bad base64,
  * SASL_BUFOVER if result won't fit,
+ * SASL_CONTINUE on a partial block,
  * SASL_OK on success
  */
 
-int sasl_decode64(const char *in, unsigned inlen,
-		  char *out, unsigned outmax, unsigned *outlen)
+int sasl_decode64(const char *in,
+                  unsigned inlen,
+                  char *out,
+                  unsigned outmax,  /* size of the buffer, not counting the NUL */
+                  unsigned *outlen)
 {
-    unsigned len = 0,lup;
-    int c1, c2, c3, c4;
+    unsigned len = 0;
+    unsigned j;
+    int c[4];
+    int saw_equal = 0;
 
     /* check parameters */
-    if (out==NULL) return SASL_FAIL;
+    if (out == NULL) return SASL_FAIL;
 
-    /* xxx these necessary? */
-    if (in[0] == '+' && in[1] == ' ') in += 2;
-    if (*in == '\r') return SASL_FAIL;
+    if (inlen > 0 && *in == '\r') return SASL_FAIL;
 
-    for (lup=0;lup<inlen/4;lup++)
-    {
-        c1 = in[0];
-        if (CHAR64(c1) == -1) return SASL_BADPROT;
-        c2 = in[1];
-        if (CHAR64(c2) == -1) return SASL_BADPROT;
-        c3 = in[2];
-        if (c3 != '=' && CHAR64(c3) == -1) return SASL_BADPROT; 
-        c4 = in[3];
-        if (c4 != '=' && CHAR64(c4) == -1) return SASL_BADPROT;
-        in += 4;
-        *out++ = (CHAR64(c1) << 2) | (CHAR64(c2) >> 4);
-        if(++len >= outmax) return SASL_BUFOVER;
-        if (c3 != '=') {
-            *out++ = ((CHAR64(c2) << 4) & 0xf0) | (CHAR64(c3) >> 2);
-            if(++len >= outmax) return SASL_BUFOVER;
-            if (c4 != '=') {
-                *out++ = ((CHAR64(c3) << 6) & 0xc0) | CHAR64(c4);
-                if(++len >= outmax) return SASL_BUFOVER;
+    while (inlen > 3) {
+        /* No data is valid after an '=' character */
+        if (saw_equal) {
+            return SASL_BADPROT;
+        }
+
+	for (j = 0; j < 4; j++) {
+	    c[j] = in[0];
+	    in++;
+	    inlen--;
+	}
+
+        if (CHAR64(c[0]) == -1 || CHAR64(c[1]) == -1) return SASL_BADPROT;
+        if (c[2] != '=' && CHAR64(c[2]) == -1) return SASL_BADPROT;
+        if (c[3] != '=' && CHAR64(c[3]) == -1) return SASL_BADPROT;
+        /* No data is valid after a '=' character, unless it is another '=' */
+        if (c[2] == '=' && c[3] != '=') return SASL_BADPROT;
+        if (c[2] == '=' || c[3] == '=') {
+            saw_equal = 1;
+        }
+
+        *out++ = (CHAR64(c[0]) << 2) | (CHAR64(c[1]) >> 4);
+        if (++len >= outmax) return SASL_BUFOVER;
+        if (c[2] != '=') {
+            *out++ = ((CHAR64(c[1]) << 4) & 0xf0) | (CHAR64(c[2]) >> 2);
+            if (++len >= outmax) return SASL_BUFOVER;
+            if (c[3] != '=') {
+                *out++ = ((CHAR64(c[2]) << 6) & 0xc0) | CHAR64(c[3]);
+                if (++len >= outmax) return SASL_BUFOVER;
             }
         }
     }
 
-    *out=0; /* terminate string */
+    if (inlen != 0) {
+        if (saw_equal) {
+            /* Unless there is CRLF at the end? */
+            return SASL_BADPROT;
+        } else {
+	    return (SASL_CONTINUE);
+        }
+    }
 
-    if(outlen) *outlen=len;
+    *out = '\0'; /* NUL terminate the output string */
+
+    if (outlen) *outlen = len;
 
     return SASL_OK;
 }
@@ -229,6 +251,7 @@ int sasl_mkchal(sasl_conn_t *conn,
 {
   sasl_rand_t *pool = NULL;
   unsigned long randnum;
+  int ret;
   time_t now;
   unsigned len;
 
@@ -236,12 +259,14 @@ int sasl_mkchal(sasl_conn_t *conn,
     + (2 * 20);			/* 2 numbers, 20 => max size of 64bit
 				 * ulong in base 10 */
   if (hostflag && conn->serverFQDN)
-    len += strlen(conn->serverFQDN) + 1 /* for the @ */;
+    len += (unsigned) strlen(conn->serverFQDN) + 1 /* for the @ */;
 
   if (maxlen < len)
     return 0;
 
-  sasl_randcreate(&pool);
+  ret = sasl_randcreate(&pool);
+  if(ret != SASL_OK) return 0; /* xxx sasl return code? */
+
   sasl_rand(pool, (char *)&randnum, sizeof(randnum));
   sasl_randfree(&pool);
 
@@ -252,7 +277,7 @@ int sasl_mkchal(sasl_conn_t *conn,
   else
     snprintf(buf,maxlen, "<%lu.%lu>", randnum, now);
 
-  return strlen(buf);
+  return (int) strlen(buf);
 }
 
   /* borrowed from larry. probably works :)
@@ -319,7 +344,13 @@ void getranddata(unsigned short ret[RPOOL_SIZE])
     {
 	struct timeval tv;
 	
-	if (!gettimeofday(&tv, NULL)) {
+	/* xxx autoconf macro */
+#ifdef _SVID_GETTOD
+	if (!gettimeofday(&tv))
+#else
+	if (!gettimeofday(&tv, NULL))
+#endif
+	{
 	    /* longs are guaranteed to be at least 32 bits; we need
 	       16 bits in each short */
 	    ret[0] ^= (unsigned short) (tv.tv_sec & 0xFFFF);
@@ -367,14 +398,17 @@ void sasl_randseed (sasl_rand_t *rpool, const char *seed, unsigned len)
     if (rpool == NULL) return;
 
     rpool->initialized = 1;
-    if (len > 6) len = sizeof(unsigned short)*RPOOL_SIZE;
+
+    if (len > sizeof(unsigned short)*RPOOL_SIZE)
+      len = sizeof(unsigned short)*RPOOL_SIZE;
+
     for (lup = 0; lup < len; lup += 2)
 	rpool->pool[lup/2] = (seed[lup] << 8) + seed[lup + 1];
 }
 
 static void randinit(sasl_rand_t *rpool)
 {
-    assert(rpool);
+    if (!rpool) return;
     
     if (!rpool->initialized) {
 	getranddata(rpool->pool);
@@ -382,7 +416,8 @@ static void randinit(sasl_rand_t *rpool)
 #if !(defined(WIN32)||defined(macintosh))
 #ifndef HAVE_JRAND48
     {
-	long *foo = (long *)rpool->pool;
+      /* xxx varies by platform */
+	unsigned int *foo = (unsigned int *)rpool->pool;
 	srandom(*foo);
     }
 #endif /* HAVE_JRAND48 */
